@@ -64,7 +64,7 @@ void db_deleter_by_type_and_id_t::delete_rows(std::string const &table,
 
         fmt::format_to(std::back_inserter(sql),
                        ") AS t (osm_type, osm_id) WHERE"
-                       " p.{} = t.osm_type AND p.{} = t.osm_id",
+                       " p.{} = t.osm_type::char(1) AND p.{} = t.osm_id",
                        type, column.c_str() + pos + 1);
     } else {
         fmt::format_to(std::back_inserter(sql),
@@ -131,9 +131,13 @@ void db_copy_thread_t::thread_t::operator()()
         // Let commits happen faster by delaying when they actually occur.
         m_conn->exec("SET synchronous_commit = off");
 
-        // Do not show messages about invalid geometries (they are removed
-        // by the trigger).
-        m_conn->exec("SET client_min_messages = WARNING");
+        // Disable sequential scan on database tables in the copy threads.
+        // The copy threads only do COPYs (which are unaffected by this
+        // setting) and DELETEs which we know benefit from the index. For
+        // some reason PostgreSQL chooses in some cases not to use that index,
+        // possibly because the DELETEs get a large list of ids to delete of
+        // which many are not in the table which confuses the query planner.
+        m_conn->exec("SET enable_seqscan = off");
 
         bool done = false;
         while (!done) {
@@ -167,7 +171,7 @@ void db_copy_thread_t::thread_t::operator()()
         m_conn.reset();
     } catch (std::runtime_error const &e) {
         log_error("DB copy thread failed: {}", e.what());
-        exit(2);
+        std::exit(2); // NOLINT(concurrency-mt-unsafe)
     }
 }
 
@@ -184,7 +188,7 @@ void db_copy_thread_t::thread_t::write_to_db(db_cmd_copy_t *buffer)
         start_copy(buffer->target);
     }
 
-    m_conn->copy_send(buffer->buffer, buffer->target->name);
+    m_conn->copy_send(buffer->buffer, buffer->target->name());
 }
 
 void db_copy_thread_t::thread_t::start_copy(
@@ -192,16 +196,16 @@ void db_copy_thread_t::thread_t::start_copy(
 {
     assert(!m_inflight);
 
-    auto const qname = qualified_name(target->schema, target->name);
+    auto const qname = qualified_name(target->schema(), target->name());
     fmt::memory_buffer sql;
-    sql.reserve(qname.size() + target->rows.size() + 20);
-    if (target->rows.empty()) {
+    sql.reserve(qname.size() + target->rows().size() + 20);
+    if (target->rows().empty()) {
         fmt::format_to(std::back_inserter(sql),
                        FMT_STRING("COPY {} FROM STDIN"), qname);
     } else {
         fmt::format_to(std::back_inserter(sql),
                        FMT_STRING("COPY {} ({}) FROM STDIN"), qname,
-                       target->rows);
+                       target->rows());
     }
 
     sql.push_back('\0');
@@ -213,7 +217,7 @@ void db_copy_thread_t::thread_t::start_copy(
 void db_copy_thread_t::thread_t::finish_copy()
 {
     if (m_inflight) {
-        m_conn->copy_end(m_inflight->name);
+        m_conn->copy_end(m_inflight->name());
         m_inflight.reset();
     }
 }
