@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -52,6 +52,17 @@ public:
      */
     virtual void way_changed(osmid_t) {}
 
+    virtual void relation_changed(osmid_t) {}
+
+    virtual void after_nodes() {}
+    virtual void after_ways() {}
+    virtual void after_relations() {}
+
+    virtual void mark_parent_relations_as_pending(
+        osmium::index::IdSetSmall<osmid_t> const & /*way_ids*/)
+    {
+    }
+
     /// Are there pending objects that need to be processed?
     virtual bool has_pending() const noexcept { return false; }
 
@@ -94,24 +105,58 @@ public:
 
     void node_changed(osmid_t id) override;
     void way_changed(osmid_t id) override;
+    void relation_changed(osmid_t id) override;
+
+    void after_nodes() override;
+    void after_ways() override;
+    void after_relations() override;
+
+    void mark_parent_relations_as_pending(
+        osmium::index::IdSetSmall<osmid_t> const &ids) override;
 
     bool has_pending() const noexcept override;
 
     idlist_t get_pending_way_ids() override
     {
-        return get_ids(m_ways_pending_tracker);
+        return get_ids(&m_ways_pending_tracker);
     }
 
     idlist_t get_pending_relation_ids() override
     {
-        return get_ids(m_rels_pending_tracker);
+        return get_ids(&m_rels_pending_tracker);
     }
 
 private:
-    static idlist_t get_ids(osmium::index::IdSetSmall<osmid_t> &tracker);
+    static idlist_t get_ids(osmium::index::IdSetSmall<osmid_t> *tracker);
 
     std::shared_ptr<middle_t> m_object_store;
 
+    /**
+     * In append mode all new and changed nodes will be added to this. After
+     * all nodes are read this is used to figure out which parent ways and
+     * relations reference these nodes. Deleted nodes are not stored in here,
+     * because all ways and relations that referenced deleted nodes must be in
+     * the change file, too, and so we don't have to find out which ones they
+     * are.
+     */
+    osmium::index::IdSetSmall<osmid_t> m_changed_nodes;
+
+    /**
+     * In append mode all new and changed ways will be added to this. After
+     * all ways are read this is used to figure out which parent relations
+     * reference these ways. Deleted ways are not stored in here, because all
+     * relations that referenced deleted ways must be in the change file, too,
+     * and so we don't have to find out which ones they are.
+     */
+    osmium::index::IdSetSmall<osmid_t> m_changed_ways;
+
+    /**
+     * In append mode all new and changed relations will be added to this.
+     * This is then used to remove already processed relations from the
+     * pending list.
+     */
+    osmium::index::IdSetSmall<osmid_t> m_changed_relations;
+
     osmium::index::IdSetSmall<osmid_t> m_ways_pending_tracker;
     osmium::index::IdSetSmall<osmid_t> m_rels_pending_tracker;
 };
