@@ -77,30 +77,35 @@ bool flex_table_t::has_id_column() const noexcept
 bool flex_table_t::matches_type(osmium::item_type type) const noexcept
 {
     // This table takes any type -> okay
-    if (m_id_type == osmium::item_type::undefined) {
+    if (m_id_type == flex_table_index_type::any_object) {
         return true;
     }
 
     // Type and table type match -> okay
-    if (type == m_id_type) {
+    if ((type == osmium::item_type::node &&
+         m_id_type == flex_table_index_type::node) ||
+        (type == osmium::item_type::way &&
+         m_id_type == flex_table_index_type::way) ||
+        (type == osmium::item_type::relation &&
+         m_id_type == flex_table_index_type::relation)) {
         return true;
     }
 
     // Relations can be written as linestrings into way tables -> okay
     if (type == osmium::item_type::relation &&
-        m_id_type == osmium::item_type::way) {
+        m_id_type == flex_table_index_type::way) {
         return true;
     }
 
     // Area tables can take ways or relations, but not nodes
-    return m_id_type == osmium::item_type::area &&
+    return m_id_type == flex_table_index_type::area &&
            type != osmium::item_type::node;
 }
 
 /// Map way/node/relation ID to id value used in database table column
 osmid_t flex_table_t::map_id(osmium::item_type type, osmid_t id) const noexcept
 {
-    if (m_id_type == osmium::item_type::undefined) {
+    if (m_id_type == flex_table_index_type::any_object) {
         if (has_multicolumn_id_index()) {
             return id;
         }
@@ -117,7 +122,7 @@ osmid_t flex_table_t::map_id(osmium::item_type type, osmid_t id) const noexcept
         }
     }
 
-    if (m_id_type != osmium::item_type::relation &&
+    if (m_id_type != flex_table_index_type::relation &&
         type == osmium::item_type::relation) {
         return -id;
     }
@@ -149,17 +154,27 @@ flex_table_column_t &flex_table_t::add_column(std::string const &name,
 
 std::string flex_table_t::build_sql_prepare_get_wkb() const
 {
-    if (has_multicolumn_id_index()) {
-        return fmt::format(
-            "PREPARE get_wkb(char(1), bigint) AS"
-            " SELECT \"{}\" FROM {} WHERE \"{}\" = $1 AND \"{}\" = $2",
-            geom_column().name(), full_name(), m_columns[0].name(),
-            m_columns[1].name());
+    util::string_joiner_t joiner{',', '"'};
+    for (auto const &column : m_columns) {
+        if (!column.expire_configs().empty()) {
+            joiner.add(column.name());
+        }
     }
 
-    return fmt::format("PREPARE get_wkb(bigint) AS"
-                       " SELECT \"{}\" FROM {} WHERE \"{}\" = $1",
-                       geom_column().name(), full_name(), id_column_names());
+    assert(!joiner.empty());
+
+    std::string const columns = joiner();
+
+    if (has_multicolumn_id_index()) {
+        return fmt::format(
+            R"(PREPARE get_wkb(char(1), bigint) AS)"
+            R"( SELECT {} FROM {} WHERE "{}" = $1 AND "{}" = $2)",
+            columns, full_name(), m_columns[0].name(), m_columns[1].name());
+    }
+
+    return fmt::format(R"(PREPARE get_wkb(bigint) AS)"
+                       R"( SELECT {} FROM {} WHERE "{}" = $1)",
+                       columns, full_name(), id_column_names());
 }
 
 std::string
@@ -220,6 +235,12 @@ flex_index_t &flex_table_t::add_index(std::string method)
     return m_indexes.emplace_back(std::move(method));
 }
 
+bool flex_table_t::has_columns_with_expire() const noexcept
+{
+    return std::any_of(m_columns.cbegin(), m_columns.cend(),
+                       [](auto const &column) { return column.has_expire(); });
+}
+
 void table_connection_t::connect(std::string const &conninfo)
 {
     assert(!m_db_connection);
@@ -256,8 +277,6 @@ void table_connection_t::start(bool append)
 {
     assert(m_db_connection);
 
-    m_db_connection->exec("SET client_min_messages = WARNING");
-
     if (!append) {
         m_db_connection->exec("DROP TABLE IF EXISTS {} CASCADE",
                               table().full_name());
@@ -265,7 +284,6 @@ void table_connection_t::start(bool append)
 
     // These _tmp tables can be left behind if we run out of disk space.
     m_db_connection->exec("DROP TABLE IF EXISTS {}", table().full_tmp_name());
-    m_db_connection->exec("RESET client_min_messages");
 
     if (!append) {
         m_db_connection->exec(table().build_sql_create_table(
@@ -298,10 +316,6 @@ void table_connection_t::stop(bool updateable, bool append)
 
         log_info("Clustering table '{}' by geometry...", table().name());
 
-        // Notices about invalid geometries are expected and can be ignored
-        // because they say nothing about the validity of the geometry in OSM.
-        m_db_connection->exec("SET client_min_messages = WARNING");
-
         m_db_connection->exec(table().build_sql_create_table(
             flex_table_t::table_type::permanent, table().full_tmp_name()));
 
@@ -359,8 +373,8 @@ void table_connection_t::stop(bool updateable, bool append)
         }
     }
 
-    if (table().always_build_id_index() ||
-        (updateable && table().has_id_column())) {
+    if ((table().always_build_id_index() || updateable) &&
+        table().has_id_column()) {
         create_id_index();
     }
 
@@ -373,7 +387,7 @@ void table_connection_t::stop(bool updateable, bool append)
 void table_connection_t::prepare()
 {
     assert(m_db_connection);
-    if (table().has_id_column() && table().has_geom_column()) {
+    if (table().has_id_column() && table().has_columns_with_expire()) {
         m_db_connection->exec(table().build_sql_prepare_get_wkb());
     }
 }
@@ -394,8 +408,8 @@ void table_connection_t::create_id_index()
     }
 }
 
-pg_result_t table_connection_t::get_geom_by_id(osmium::item_type type,
-                                               osmid_t id) const
+pg_result_t table_connection_t::get_geoms_by_id(osmium::item_type type,
+                                                osmid_t id) const
 {
     assert(table().has_geom_column());
     assert(m_db_connection);
@@ -421,4 +435,7 @@ void table_connection_t::task_wait()
     auto const run_time = m_task_result.wait();
     log_info("All postprocessing on table '{}' done in {}.", table().name(),
              util::human_readable_duration(run_time));
+    log_debug("Inserted {} rows into table '{}' ({} not inserted due to"
+              " NOT NULL constraints).",
+              m_count_insert, table().name(), m_count_not_null_error);
 }
