@@ -13,8 +13,6 @@
 /**
  * \file
  *
- * This file is part of osm2pgsql (https://github.com/openstreetmap/osm2pgsql).
- *
  * It contains the functions reading and checking the input data.
  */
 
@@ -23,6 +21,7 @@
 
 #include <osmium/fwd.hpp>
 #include <osmium/io/file.hpp>
+#include <osmium/io/header.hpp>
 
 #include "osmtypes.hpp"
 
@@ -34,6 +33,12 @@ struct type_id
     osmid_t id;
 };
 
+struct file_info
+{
+    osmium::io::Header header{};
+    osmium::Timestamp last_timestamp{};
+};
+
 /**
  * Compare two tuples (type, id). Throw a descriptive error if either the
  * curr id is negative or if the data is not ordered.
@@ -53,7 +58,7 @@ prepare_input_files(std::vector<std::string> const &input_files,
 /**
  * Process the specified OSM files (stage 1a).
  */
-void process_files(std::vector<osmium::io::File> const &files,
-                   osmdata_t *osmdata, bool append, bool show_progress);
+file_info process_files(std::vector<osmium::io::File> const &files,
+                        osmdata_t *osmdata, bool append, bool show_progress);
 
 #endif // OSM2PGSQL_INPUT_HPP
