@@ -18,6 +18,7 @@
  * emit the final geometry-enabled output formats
 */
 
+#include <map>
 #include <memory>
 
 #include <osmium/index/nwr_array.hpp>
@@ -28,14 +29,35 @@
 
 class node_locations_t;
 class node_persistent_cache;
-class options_t;
+
+struct middle_pgsql_options
+{
+    // Store nodes in database.
+    bool nodes = false;
+
+    // Store untagged nodes also (set in addition to nodes=true).
+    bool untagged_nodes = false;
+
+    // Bit shift used in way node index
+    uint8_t way_node_index_id_shift = 5;
+
+    // Database format (legacy = 1, new = 2)
+    uint8_t db_format = 1;
+
+    // Use a flat node file
+    bool use_flat_node_file = false;
+
+    // Store attributes (timestamp, version, changeset id, user id, user name)
+    bool with_attributes = false;
+};
 
 class middle_query_pgsql_t : public middle_query_t
 {
 public:
     middle_query_pgsql_t(
         std::string const &conninfo, std::shared_ptr<node_locations_t> cache,
-        std::shared_ptr<node_persistent_cache> persistent_cache);
+        std::shared_ptr<node_persistent_cache> persistent_cache,
+        middle_pgsql_options const &options);
 
     osmium::Location get_node_location(osmid_t id) const override;
 
@@ -47,6 +69,9 @@ public:
                            osmium::memory::Buffer *buffer,
                            osmium::osm_entity_bits::type types) const override;
 
+    bool relation_get_format1(osmid_t id, osmium::memory::Buffer *buffer) const;
+    bool relation_get_format2(osmid_t id, osmium::memory::Buffer *buffer) const;
+
     bool relation_get(osmid_t id,
                       osmium::memory::Buffer *buffer) const override;
 
@@ -61,14 +86,16 @@ private:
     pg_conn_t m_sql_conn;
     std::shared_ptr<node_locations_t> m_cache;
     std::shared_ptr<node_persistent_cache> m_persistent_cache;
+
+    middle_pgsql_options m_store_options;
 };
 
-struct table_sql {
-    char const *name = "";
-    char const *create_table = "";
-    char const *prepare_query = "";
-    char const *prepare_fw_dep_lookups = "";
-    char const *create_fw_dep_indexes = "";
+struct table_sql
+{
+    std::string name;
+    std::string create_table;
+    std::vector<std::string> prepare_queries;
+    std::vector<std::string> create_fw_dep_indexes;
 };
 
 struct middle_pgsql_t : public middle_t
@@ -89,9 +116,14 @@ struct middle_pgsql_t : public middle_t
     void after_ways() override;
     void after_relations() override;
 
-    idlist_t get_ways_by_node(osmid_t osm_id) override;
-    idlist_t get_rels_by_node(osmid_t osm_id) override;
-    idlist_t get_rels_by_way(osmid_t osm_id) override;
+    void get_node_parents(
+        osmium::index::IdSetSmall<osmid_t> const &changed_nodes,
+        osmium::index::IdSetSmall<osmid_t> *parent_ways,
+        osmium::index::IdSetSmall<osmid_t> *parent_relations) const override;
+
+    void get_way_parents(
+        osmium::index::IdSetSmall<osmid_t> const &changed_ways,
+        osmium::index::IdSetSmall<osmid_t> *parent_relations) const override;
 
     class table_desc
     {
@@ -101,10 +133,13 @@ struct middle_pgsql_t : public middle_t
 
         std::string const &schema() const noexcept
         {
-            return m_copy_target->schema;
+            return m_copy_target->schema();
         }
 
-        std::string const &name() const noexcept { return m_copy_target->name; }
+        std::string const &name() const noexcept
+        {
+            return m_copy_target->name();
+        }
 
         std::shared_ptr<db_target_descr_t> const &copy_target() const noexcept
         {
@@ -118,9 +153,8 @@ struct middle_pgsql_t : public middle_t
         void build_index(std::string const &conninfo) const;
 
         std::string m_create_table;
-        std::string m_prepare_query;
-        std::string m_prepare_fw_dep_lookups;
-        std::string m_create_fw_dep_indexes;
+        std::vector<std::string> m_prepare_queries;
+        std::vector<std::string> m_create_fw_dep_indexes;
 
         void task_set(std::future<std::chrono::microseconds> &&future)
         {
@@ -129,13 +163,22 @@ struct middle_pgsql_t : public middle_t
 
         std::chrono::microseconds task_wait() { return m_task_result.wait(); }
 
+        void init_max_id(pg_conn_t const &db_connection);
+
+        osmid_t max_id() const noexcept { return m_max_id; }
+
     private:
         std::shared_ptr<db_target_descr_t> m_copy_target;
         task_result_t m_task_result;
+
+        /// The maximum id in the table (used only in append mode)
+        osmid_t m_max_id = 0;
     };
 
     std::shared_ptr<middle_query_t> get_query_instance() override;
 
+    void set_requirements(output_requirements const &requirements) override;
+
 private:
     void node_set(osmium::Node const &node);
     void node_delete(osmid_t id);
@@ -143,12 +186,23 @@ private:
     void way_set(osmium::Way const &way);
     void way_delete(osmid_t id);
 
+    void relation_set_format1(osmium::Relation const &rel);
+    void relation_set_format2(osmium::Relation const &rel);
+
     void relation_set(osmium::Relation const &rel);
     void relation_delete(osmid_t id);
 
     void buffer_store_tags(osmium::OSMObject const &obj, bool attrs);
 
+    void copy_attributes(osmium::OSMObject const &obj);
+    void copy_tags(osmium::OSMObject const &obj);
+
+    void write_users_table();
+    void update_users_table();
+
+    std::map<osmium::user_id_type, std::string> m_users;
     osmium::nwr_array<table_desc> m_tables;
+    table_desc m_users_table;
 
     options_t const *m_options;
 
@@ -160,6 +214,11 @@ private:
     // middle keeps its own thread for writing to the database.
     std::shared_ptr<db_copy_thread_t> m_copy_thread;
     db_copy_mgr_t<db_deleter_by_id_t> m_db_copy;
+
+    /// Options for this middle.
+    middle_pgsql_options m_store_options;
+
+    bool m_append;
 };
 
 #endif // OSM2PGSQL_MIDDLE_PGSQL_HPP
