@@ -85,7 +85,7 @@ private:
 
     /// The maximum number of bytes an entry will need in storage.
     constexpr static std::size_t max_bytes_per_entry() noexcept {
-        return 10U /*max varint length*/ * 3U /*id, x, y*/;
+        return 10UL /*max varint length*/ * 3UL /*id, x, y*/;
     }
 
     bool will_resize() const noexcept
