@@ -19,6 +19,13 @@
 
 class reprojection;
 
+enum class command_t
+{
+    help,
+    version,
+    process
+};
+
 /// Variants for generation of hstore column
 enum class hstore_column : char
 {
@@ -70,31 +77,14 @@ struct output_requirements
 /**
  * Structure for storing command-line and other options
  */
-class options_t
+struct options_t
 {
-public:
-    /**
-     * Constructor setting default values for all options. Used for testing.
-     */
-    options_t();
-
-    /**
-     * Constructor parsing the options from the command line.
-     */
-    // NOLINTNEXTLINE(cppcoreguidelines-avoid-c-arrays,hicpp-avoid-c-arrays,modernize-avoid-c-arrays)
-    options_t(int argc, char *argv[]);
-
-    /**
-     * Return true if the main program should end directly after the option
-     * parsing. This is true when a help text was printed.
-     */
-    bool early_return() const noexcept {
-        return m_print_help;
-    }
+    command_t command = command_t::process;
 
     std::string conninfo; ///< connection info for database
 
     std::string prefix{"planet_osm"};         ///< prefix for table names
+    bool prefix_is_set = false;
 
     /// Pg Tablespace to store indexes on main tables (no default TABLESPACE)
     std::string tblsmain_index{};
@@ -108,13 +98,16 @@ public:
     /// Pg Tablespace to store slim tables (no default TABLESPACE)
     std::string tblsslim_data{};
 
-    /// Pg schema to store middle tables in, default none
+    /// Default Pg schema.
+    std::string dbschema{"public"};
+
+    /// Pg schema to store middle tables in.
     std::string middle_dbschema{};
 
-    /// Pg schema to store output tables in, default none
+    /// Pg schema to store output tables in.
     std::string output_dbschema{};
 
-    std::string style{DEFAULT_STYLE}; ///< style file to use
+    std::string style{}; ///< style file to use
 
     /// Name of the flat node file used. Empty if flat node file is not enabled.
     std::string flat_node_file{};
@@ -146,7 +139,7 @@ public:
 
     int cache = 800; ///< Memory usable for cache in MB
 
-    unsigned int num_procs;
+    unsigned int num_procs = 1;
 
     /**
      * How many bits should the node id be shifted for the way node index?
@@ -156,6 +149,16 @@ public:
      */
     uint8_t way_node_index_id_shift = 5;
 
+    /// Database format (0=unknown/no database middle, 1=legacy, 2=new)
+    uint8_t middle_database_format = 1;
+
+    /**
+     * Should nodes (with tags) be stored in the middle? If no flat node file
+     * is used, nodes will always be stored. (Only works with the new middle
+     * database format.)
+     */
+    bool middle_with_nodes = false;
+
     /// add an additional hstore column with objects key/value pairs, and what type of hstore column
     hstore_column hstore_mode = hstore_column::none;
 
@@ -185,14 +188,8 @@ public:
     bool create = false;
     bool pass_prompt = false;
 
-private:
-
-    bool m_print_help = false;
-
-    /**
-     * Check input options for sanity
-     */
-    void check_options();
-};
+    bool output_backend_set = false;
+    bool style_set = false;
+}; // struct options_t
 
 #endif // OSM2PGSQL_OPTIONS_HPP
