@@ -13,8 +13,6 @@
 /**
  * \file
  *
- * This file is part of osm2pgsql (https://github.com/openstreetmap/osm2pgsql).
- *
  * It contains the osmdata_t class.
  */
 
@@ -29,8 +27,8 @@
 #include "osmtypes.hpp"
 
 class middle_t;
-class options_t;
 class output_t;
+struct options_t;
 
 /**
  * This class guides the processing of the OSM data through its multiple
@@ -61,18 +59,6 @@ public:
     void stop() const;
 
 private:
-    void node_add(osmium::Node const &node) const;
-    void way_add(osmium::Way *way) const;
-    void relation_add(osmium::Relation const &rel) const;
-
-    void node_modify(osmium::Node const &node) const;
-    void way_modify(osmium::Way *way) const;
-    void relation_modify(osmium::Relation const &rel) const;
-
-    void node_delete(osmid_t id) const;
-    void way_delete(osmid_t id) const;
-    void relation_delete(osmid_t id) const;
-
     /**
      * Run stage 1b and stage 1c processing: Process dependent objects in
      * append mode.
