@@ -8,10 +8,14 @@
  */
 
 #include "db-copy.hpp"
+#include "debug-output.hpp"
+#include "expire-output.hpp"
 #include "expire-tiles.hpp"
 #include "flex-index.hpp"
 #include "flex-lua-geom.hpp"
 #include "flex-lua-index.hpp"
+#include "flex-lua-table.hpp"
+#include "flex-lua-expire-output.hpp"
 #include "flex-write.hpp"
 #include "format.hpp"
 #include "geom-from-osm.hpp"
@@ -19,6 +23,7 @@
 #include "geom-transform.hpp"
 #include "logging.hpp"
 #include "lua-init.hpp"
+#include "lua-setup.hpp"
 #include "lua-utils.hpp"
 #include "middle.hpp"
 #include "options.hpp"
@@ -30,17 +35,10 @@
 #include "thread-pool.hpp"
 #include "util.hpp"
 #include "version.hpp"
+#include "wkb.hpp"
 
 #include <osmium/osm/types_from_string.hpp>
 
-extern "C"
-{
-#include <lauxlib.h>
-#include <lualib.h>
-}
-
-#include <boost/filesystem.hpp>
-
 #include <cassert>
 #include <cstdlib>
 #include <cstring>
@@ -50,9 +48,6 @@ extern "C"
 #include <stdexcept>
 #include <string>
 
-// Mutex used to coordinate access to Lua code
-static std::mutex lua_mutex;
-
 // Lua can't call functions on C++ objects directly. This macro defines simple
 // C "trampoline" functions which are called from Lua which get the current
 // context (the output_flex_t object) and call the respective function on the
@@ -74,6 +69,7 @@ static std::mutex lua_mutex;
     }
 
 TRAMPOLINE(app_define_table, define_table)
+TRAMPOLINE(app_define_expire_output, define_expire_output)
 TRAMPOLINE(app_get_bbox, get_bbox)
 
 TRAMPOLINE(app_as_point, as_point)
@@ -92,7 +88,13 @@ TRAMPOLINE(table_insert, insert)
 TRAMPOLINE(table_columns, columns)
 TRAMPOLINE(table_tostring, __tostring)
 
-static char const *const osm2pgsql_table_name = "osm2pgsql.Table";
+TRAMPOLINE(expire_output_minzoom, minzoom)
+TRAMPOLINE(expire_output_maxzoom, maxzoom)
+TRAMPOLINE(expire_output_filename, filename)
+TRAMPOLINE(expire_output_schema, schema)
+TRAMPOLINE(expire_output_table, table)
+TRAMPOLINE(expire_output_tostring, __tostring)
+
 static char const *const osm2pgsql_object_metatable =
     "osm2pgsql.object_metatable";
 
@@ -142,63 +144,19 @@ static void push_osm_object_to_lua_stack(lua_State *lua_state,
     if (with_attributes) {
         if (object.version() != 0U) {
             luaX_add_table_int(lua_state, "version", object.version());
-        } else {
-            // This is a workaround, because the middle will give us the
-            // attributes as pseudo-tags.
-            char const *const val = object.tags()["osm_version"];
-            if (val) {
-                luaX_add_table_int(lua_state, "version",
-                                   osmium::string_to_object_version(val));
-            }
         }
-
         if (object.timestamp().valid()) {
             luaX_add_table_int(lua_state, "timestamp",
                                object.timestamp().seconds_since_epoch());
-        } else {
-            // This is a workaround, because the middle will give us the
-            // attributes as pseudo-tags.
-            char const *const val = object.tags()["osm_timestamp"];
-            if (val) {
-                auto const timestamp = osmium::Timestamp{val};
-                luaX_add_table_int(lua_state, "timestamp",
-                                   timestamp.seconds_since_epoch());
-            }
         }
-
         if (object.changeset() != 0U) {
             luaX_add_table_int(lua_state, "changeset", object.changeset());
-        } else {
-            char const *const val = object.tags()["osm_changeset"];
-            // This is a workaround, because the middle will give us the
-            // attributes as pseudo-tags.
-            if (val) {
-                luaX_add_table_int(lua_state, "changeset",
-                                   osmium::string_to_changeset_id(val));
-            }
         }
-
         if (object.uid() != 0U) {
             luaX_add_table_int(lua_state, "uid", object.uid());
-        } else {
-            // This is a workaround, because the middle will give us the
-            // attributes as pseudo-tags.
-            char const *const val = object.tags()["osm_uid"];
-            if (val) {
-                luaX_add_table_int(lua_state, "uid",
-                                   osmium::string_to_uid(val));
-            }
         }
-
         if (object.user()[0] != '\0') {
             luaX_add_table_str(lua_state, "user", object.user());
-        } else {
-            // This is a workaround, because the middle will give us the
-            // attributes as pseudo-tags.
-            char const *const val = object.tags()["osm_user"];
-            if (val) {
-                luaX_add_table_str(lua_state, "user", val);
-            }
         }
     }
 
@@ -435,253 +393,6 @@ int output_flex_t::app_as_geometrycollection()
     return 1;
 }
 
-static void check_tablespace(std::string const &tablespace)
-{
-    if (!has_tablespace(tablespace)) {
-        throw fmt_error(
-            "Tablespace '{0}' not available."
-            " Use 'CREATE TABLESPACE \"{0}\" ...;' to create it.",
-            tablespace);
-    }
-}
-
-flex_table_t &output_flex_t::create_flex_table()
-{
-    std::string const table_name =
-        luaX_get_table_string(lua_state(), "name", -1, "The table");
-
-    check_identifier(table_name, "table names");
-
-    if (util::find_by_name(*m_tables, table_name)) {
-        throw fmt_error("Table with name '{}' already exists.", table_name);
-    }
-
-    auto &new_table = m_tables->emplace_back(table_name);
-
-    lua_pop(lua_state(), 1); // "name"
-
-    // optional "schema" field
-    lua_getfield(lua_state(), -1, "schema");
-    if (lua_isstring(lua_state(), -1)) {
-        std::string const schema = lua_tostring(lua_state(), -1);
-        check_identifier(schema, "schema field");
-        if (!has_schema(schema)) {
-            throw fmt_error("Schema '{0}' not available."
-                            " Use 'CREATE SCHEMA \"{0}\";' to create it.",
-                            schema);
-        }
-        new_table.set_schema(schema);
-    }
-    lua_pop(lua_state(), 1);
-
-    // optional "cluster" field
-    lua_getfield(lua_state(), -1, "cluster");
-    int const cluster_type = lua_type(lua_state(), -1);
-    if (cluster_type == LUA_TSTRING) {
-        std::string const cluster = lua_tostring(lua_state(), -1);
-        if (cluster == "auto") {
-            new_table.set_cluster_by_geom(true);
-        } else if (cluster == "no") {
-            new_table.set_cluster_by_geom(false);
-        } else {
-            throw fmt_error("Unknown value '{}' for 'cluster' table option"
-                            " (use 'auto' or 'no').",
-                            cluster);
-        }
-    } else if (cluster_type == LUA_TNIL) {
-        // ignore
-    } else {
-        throw std::runtime_error{
-            "Unknown value for 'cluster' table option: Must be string."};
-    }
-    lua_pop(lua_state(), 1);
-
-    // optional "data_tablespace" field
-    lua_getfield(lua_state(), -1, "data_tablespace");
-    if (lua_isstring(lua_state(), -1)) {
-        std::string const tablespace = lua_tostring(lua_state(), -1);
-        check_identifier(tablespace, "data_tablespace field");
-        check_tablespace(tablespace);
-        new_table.set_data_tablespace(tablespace);
-    }
-    lua_pop(lua_state(), 1);
-
-    // optional "index_tablespace" field
-    lua_getfield(lua_state(), -1, "index_tablespace");
-    if (lua_isstring(lua_state(), -1)) {
-        std::string const tablespace = lua_tostring(lua_state(), -1);
-        check_identifier(tablespace, "index_tablespace field");
-        check_tablespace(tablespace);
-        new_table.set_index_tablespace(tablespace);
-    }
-    lua_pop(lua_state(), 1);
-
-    return new_table;
-}
-
-void output_flex_t::setup_id_columns(flex_table_t *table)
-{
-    assert(table);
-    lua_getfield(lua_state(), -1, "ids");
-    if (lua_type(lua_state(), -1) != LUA_TTABLE) {
-        log_warn("Table '{}' doesn't have an id column. Two-stage"
-                 " processing, updates and expire will not work!",
-                 table->name());
-        lua_pop(lua_state(), 1); // ids
-        return;
-    }
-
-    std::string const type{
-        luaX_get_table_string(lua_state(), "type", -1, "The ids field")};
-    lua_pop(lua_state(), 1); // "type"
-
-    if (type == "node") {
-        table->set_id_type(osmium::item_type::node);
-    } else if (type == "way") {
-        table->set_id_type(osmium::item_type::way);
-    } else if (type == "relation") {
-        table->set_id_type(osmium::item_type::relation);
-    } else if (type == "area") {
-        table->set_id_type(osmium::item_type::area);
-    } else if (type == "any") {
-        table->set_id_type(osmium::item_type::undefined);
-        lua_getfield(lua_state(), -1, "type_column");
-        if (lua_isstring(lua_state(), -1)) {
-            std::string const column_name =
-                lua_tolstring(lua_state(), -1, nullptr);
-            check_identifier(column_name, "column names");
-            auto &column = table->add_column(column_name, "id_type", "");
-            column.set_not_null();
-        } else if (!lua_isnil(lua_state(), -1)) {
-            throw std::runtime_error{"type_column must be a string or nil."};
-        }
-        lua_pop(lua_state(), 1); // "type_column"
-    } else {
-        throw fmt_error("Unknown ids type: {}.", type);
-    }
-
-    std::string const name =
-        luaX_get_table_string(lua_state(), "id_column", -1, "The ids field");
-    lua_pop(lua_state(), 1); // "id_column"
-    check_identifier(name, "column names");
-
-    std::string const create_index = luaX_get_table_string(
-        lua_state(), "create_index", -1, "The ids field", "auto");
-    lua_pop(lua_state(), 1); // "create_index"
-    if (create_index == "always") {
-        table->set_always_build_id_index();
-    } else if (create_index != "auto") {
-        throw fmt_error("Unknown value '{}' for 'create_index' field of ids",
-                        create_index);
-    }
-
-    auto &column = table->add_column(name, "id_num", "");
-    column.set_not_null();
-    lua_pop(lua_state(), 1); // "ids"
-}
-
-void output_flex_t::setup_flex_table_columns(flex_table_t *table)
-{
-    assert(table);
-    lua_getfield(lua_state(), -1, "columns");
-    if (lua_type(lua_state(), -1) != LUA_TTABLE) {
-        throw fmt_error("No 'columns' field (or not an array) in table '{}'.",
-                        table->name());
-    }
-
-    if (!luaX_is_array(lua_state())) {
-        throw std::runtime_error{"The 'columns' field must contain an array."};
-    }
-    std::size_t num_columns = 0;
-    luaX_for_each(lua_state(), [&]() {
-        if (!lua_istable(lua_state(), -1)) {
-            throw std::runtime_error{
-                "The entries in the 'columns' array must be tables."};
-        }
-
-        char const *const type = luaX_get_table_string(lua_state(), "type", -1,
-                                                       "Column entry", "text");
-        char const *const name =
-            luaX_get_table_string(lua_state(), "column", -2, "Column entry");
-        check_identifier(name, "column names");
-        char const *const sql_type = luaX_get_table_string(
-            lua_state(), "sql_type", -3, "Column entry", "");
-
-        auto &column = table->add_column(name, type, sql_type);
-        lua_pop(lua_state(), 3); // "type", "column", "sql_type"
-
-        column.set_not_null(luaX_get_table_bool(lua_state(), "not_null", -1,
-                                                "Entry 'not_null'", false));
-        lua_pop(lua_state(), 1); // "not_null"
-
-        column.set_create_only(luaX_get_table_bool(
-            lua_state(), "create_only", -1, "Entry 'create_only'", false));
-        lua_pop(lua_state(), 1); // "create_only"
-
-        lua_getfield(lua_state(), -1, "projection");
-        if (!lua_isnil(lua_state(), -1)) {
-            if (column.is_geometry_column() ||
-                column.type() == table_column_type::area) {
-                column.set_projection(lua_tostring(lua_state(), -1));
-            } else {
-                throw std::runtime_error{"Projection can only be set on "
-                                         "geometry and area columns."};
-            }
-        }
-        lua_pop(lua_state(), 1); // "projection"
-
-        ++num_columns;
-    });
-
-    if (num_columns == 0 && !table->has_id_column()) {
-        throw fmt_error("No columns defined for table '{}'.", table->name());
-    }
-
-    lua_pop(lua_state(), 1); // "columns"
-}
-
-void output_flex_t::setup_indexes(flex_table_t *table)
-{
-    assert(table);
-
-    lua_getfield(lua_state(), -1, "indexes");
-    if (lua_type(lua_state(), -1) == LUA_TNIL) {
-        if (table->has_geom_column()) {
-            auto &index = table->add_index("gist");
-            index.set_columns(table->geom_column().name());
-
-            if (!get_options()->slim || get_options()->droptemp) {
-                // If database can not be updated, use fillfactor 100.
-                index.set_fillfactor(100);
-            }
-            index.set_tablespace(table->index_tablespace());
-        }
-        lua_pop(lua_state(), 1); // "indexes"
-        return;
-    }
-
-    if (lua_type(lua_state(), -1) != LUA_TTABLE) {
-        throw fmt_error("The 'indexes' field in definition of"
-                        " table '{}' is not an array.",
-                        table->name());
-    }
-
-    if (!luaX_is_array(lua_state())) {
-        throw std::runtime_error{"The 'indexes' field must contain an array."};
-    }
-
-    luaX_for_each(lua_state(), [&]() {
-        if (!lua_istable(lua_state(), -1)) {
-            throw std::runtime_error{
-                "The entries in the 'indexes' array must be Lua tables."};
-        }
-
-        flex_lua_setup_index(lua_state(), table);
-    });
-
-    lua_pop(lua_state(), 1); // "indexes"
-}
-
 int output_flex_t::app_define_table()
 {
     if (m_calling_context != calling_context::main) {
@@ -690,57 +401,69 @@ int output_flex_t::app_define_table()
             " main Lua code, not in any of the callbacks."};
     }
 
-    if (lua_type(lua_state(), 1) != LUA_TTABLE) {
-        throw std::runtime_error{
-            "Argument #1 to 'define_table' must be a table."};
-    }
-
-    auto &new_table = create_flex_table();
-    setup_id_columns(&new_table);
-    setup_flex_table_columns(&new_table);
-    setup_indexes(&new_table);
-
-    void *ptr = lua_newuserdata(lua_state(), sizeof(std::size_t));
-    auto *num = new (ptr) std::size_t{};
-    *num = m_tables->size() - 1;
-    luaL_getmetatable(lua_state(), osm2pgsql_table_name);
-    lua_setmetatable(lua_state(), -2);
-
-    return 1;
+    return setup_flex_table(lua_state(), m_tables.get(), m_expire_outputs.get(),
+                            get_options()->dbschema,
+                            get_options()->slim && !get_options()->droptemp,
+                            get_options()->append);
 }
 
-// Check that the first element on the Lua stack is an osm2pgsql.Table
-// parameter and return its internal table index.
-static std::size_t table_idx_from_param(lua_State *lua_state)
+int output_flex_t::app_define_expire_output()
 {
+    if (m_calling_context != calling_context::main) {
+        throw std::runtime_error{
+            "Expire outputs have to be defined in the"
+            " main Lua code, not in any of the callbacks."};
+    }
+
+    return setup_flex_expire_output(lua_state(), get_options()->dbschema,
+                                    m_expire_outputs.get());
+}
+
+// Check that the first element on the Lua stack is a "type_name"
+// parameter and return its internal index.
+static std::size_t idx_from_param(lua_State *lua_state, char const *type_name)
+{
+    assert(lua_gettop(lua_state) >= 1);
+
     void const *const user_data = lua_touserdata(lua_state, 1);
 
     if (user_data == nullptr || !lua_getmetatable(lua_state, 1)) {
-        throw std::runtime_error{
-            "First parameter must be of type osm2pgsql.Table."};
+        throw fmt_error("First parameter must be of type {}.", type_name);
     }
 
-    luaL_getmetatable(lua_state, osm2pgsql_table_name);
+    luaL_getmetatable(lua_state, type_name);
     if (!lua_rawequal(lua_state, -1, -2)) {
-        throw std::runtime_error{
-            "First parameter must be of type osm2pgsql.Table."};
+        throw fmt_error("First parameter must be of type {}.", type_name);
     }
-    lua_pop(lua_state, 2);
+    lua_pop(lua_state, 2); // remove the two metatables
 
     return *static_cast<std::size_t const *>(user_data);
 }
 
-// Get the flex table that is as first parameter on the Lua stack.
-flex_table_t const &output_flex_t::get_table_from_param()
+template <typename CONTAINER>
+static typename CONTAINER::value_type const &
+get_from_idx_param(lua_State *lua_state, CONTAINER *container,
+                   char const *type_name)
 {
-    if (lua_gettop(lua_state()) != 1) {
-        throw std::runtime_error{
-            "Need exactly one parameter of type osm2pgsql.Table."};
+    if (lua_gettop(lua_state) != 1) {
+        throw fmt_error("Need exactly one parameter of type {}.", type_name);
     }
 
-    auto const &table = m_tables->at(table_idx_from_param(lua_state()));
-    lua_remove(lua_state(), 1);
-    return table;
+    auto const &item = container->at(idx_from_param(lua_state, type_name));
+    lua_remove(lua_state, 1);
+    return item;
+}
+
+flex_table_t const &output_flex_t::get_table_from_param()
+{
+    return get_from_idx_param(lua_state(), m_tables.get(),
+                              osm2pgsql_table_name);
+}
+
+expire_output_t const &output_flex_t::get_expire_output_from_param()
+{
+    return get_from_idx_param(lua_state(), m_expire_outputs.get(),
+                              osm2pgsql_expire_output_name);
 }
 
 int output_flex_t::table_tostring()
@@ -850,8 +573,9 @@ int output_flex_t::table_add_row()
             "Need two parameters: The osm2pgsql.Table and the row data."};
     }
 
-    auto &table_connection =
-        m_table_connections.at(table_idx_from_param(lua_state()));
+    auto &table_connection = m_table_connections.at(
+        idx_from_param(lua_state(), osm2pgsql_table_name));
+
     auto const &table = table_connection.table();
 
     // If there is a second parameter, it must be a Lua table.
@@ -927,8 +651,8 @@ int output_flex_t::table_insert()
     }
 
     // The first parameter is the table object.
-    auto &table_connection =
-        m_table_connections.at(table_idx_from_param(lua_state()));
+    auto &table_connection = m_table_connections.at(
+        idx_from_param(lua_state(), osm2pgsql_table_name));
 
     // The second parameter must be a Lua table with the contents for the
     // fields.
@@ -952,9 +676,11 @@ int output_flex_t::table_insert()
             } else if (column.type() == table_column_type::id_num) {
                 copy_mgr->add_column(id);
             } else {
-                flex_write_column(lua_state(), copy_mgr, column, &m_expire);
+                flex_write_column(lua_state(), copy_mgr, column,
+                                  &m_expire_tiles);
             }
         }
+        table_connection.increment_insert_counter();
     } catch (not_null_exception const &e) {
         copy_mgr->rollback_line();
         lua_pushboolean(lua_state(), false);
@@ -962,6 +688,7 @@ int output_flex_t::table_insert()
         lua_pushstring(lua_state(), e.column().name().c_str());
         push_osm_object_to_lua_stack(lua_state(), object,
                                      get_options()->extra_attributes);
+        table_connection.increment_not_null_error_counter();
         return 4;
     }
 
@@ -1065,7 +792,7 @@ void output_flex_t::add_row(table_connection_t *table_connection,
 
     if (!table.has_geom_column()) {
         flex_write_row(lua_state(), table_connection, object.type(), id, {}, 0,
-                       &m_expire);
+                       &m_expire_tiles);
         return;
     }
 
@@ -1103,15 +830,74 @@ void output_flex_t::add_row(table_connection_t *table_connection,
 
     auto const geoms = geom::split_multi(std::move(geom), split_multi);
     for (auto const &sgeom : geoms) {
-        m_expire.from_geometry_if_3857(sgeom);
+        table.geom_column().do_expire(sgeom, &m_expire_tiles);
         flex_write_row(lua_state(), table_connection, object.type(), id, sgeom,
-                       table.geom_column().srid(), &m_expire);
+                       table.geom_column().srid(), &m_expire_tiles);
+        table_connection->increment_insert_counter();
     }
 }
 
+int output_flex_t::expire_output_tostring()
+{
+    auto const &expire_output = get_expire_output_from_param();
+
+    std::string const str =
+        fmt::format("osm2pgsql.ExpireOutput[minzoom={},maxzoom={},filename={},"
+                    "schema={},table={}]",
+                    expire_output.minzoom(), expire_output.maxzoom(),
+                    expire_output.filename(), expire_output.schema(),
+                    expire_output.table());
+    lua_pushstring(lua_state(), str.c_str());
+
+    return 1;
+}
+
+int output_flex_t::expire_output_minzoom()
+{
+    auto const &expire_output = get_expire_output_from_param();
+
+    lua_pushinteger(lua_state(), expire_output.minzoom());
+    return 1;
+}
+
+int output_flex_t::expire_output_maxzoom()
+{
+    auto const &expire_output = get_expire_output_from_param();
+
+    lua_pushinteger(lua_state(), expire_output.maxzoom());
+    return 1;
+}
+
+int output_flex_t::expire_output_filename()
+{
+    auto const &expire_output = get_expire_output_from_param();
+
+    lua_pushstring(lua_state(), expire_output.filename().c_str());
+    return 1;
+}
+
+int output_flex_t::expire_output_schema()
+{
+    auto const &expire_output = get_expire_output_from_param();
+
+    lua_pushstring(lua_state(), expire_output.schema().c_str());
+    return 1;
+}
+
+int output_flex_t::expire_output_table()
+{
+    auto const &expire_output = get_expire_output_from_param();
+
+    lua_pushstring(lua_state(), expire_output.table().c_str());
+    return 1;
+}
+
 void output_flex_t::call_lua_function(prepared_lua_function_t func,
                                       osmium::OSMObject const &object)
 {
+    static std::mutex lua_mutex;
+    std::lock_guard<std::mutex> const guard{lua_mutex};
+
     m_calling_context = func.context();
 
     lua_pushvalue(lua_state(), func.index()); // the function to call
@@ -1128,13 +914,6 @@ void output_flex_t::call_lua_function(prepared_lua_function_t func,
     m_calling_context = calling_context::main;
 }
 
-void output_flex_t::get_mutex_and_call_lua_function(
-    prepared_lua_function_t func, osmium::OSMObject const &object)
-{
-    std::lock_guard<std::mutex> const guard{lua_mutex};
-    call_lua_function(func, object);
-}
-
 void output_flex_t::pending_way(osmid_t id)
 {
     if (!m_process_way) {
@@ -1147,7 +926,7 @@ void output_flex_t::pending_way(osmid_t id)
 
     way_delete(id);
 
-    get_mutex_and_call_lua_function(m_process_way, m_way_cache.get());
+    call_lua_function(m_process_way, m_way_cache.get());
 }
 
 void output_flex_t::select_relation_members()
@@ -1156,7 +935,6 @@ void output_flex_t::select_relation_members()
         return;
     }
 
-    std::lock_guard<std::mutex> const guard{lua_mutex};
     call_lua_function(m_select_relation_members, m_relation_cache.get());
 
     // If the function returned nil there is nothing to be marked.
@@ -1236,8 +1014,7 @@ void output_flex_t::pending_relation(osmid_t id)
     delete_from_tables(osmium::item_type::relation, id);
 
     if (m_process_relation) {
-        get_mutex_and_call_lua_function(m_process_relation,
-                                        m_relation_cache.get());
+        call_lua_function(m_process_relation, m_relation_cache.get());
     }
 }
 
@@ -1252,7 +1029,7 @@ void output_flex_t::pending_relation_stage1c(osmid_t id)
     }
 
     m_disable_add_row = true;
-    get_mutex_and_call_lua_function(m_process_relation, m_relation_cache.get());
+    call_lua_function(m_process_relation, m_relation_cache.get());
     m_disable_add_row = false;
 }
 
@@ -1263,6 +1040,20 @@ void output_flex_t::sync()
     }
 }
 
+void output_flex_t::after_nodes()
+{
+    for (auto &table : m_table_connections) {
+        table.flush();
+    }
+}
+
+void output_flex_t::after_ways()
+{
+    for (auto &table : m_table_connections) {
+        table.flush();
+    }
+}
+
 void output_flex_t::stop()
 {
     for (auto &table : m_table_connections) {
@@ -1272,19 +1063,37 @@ void output_flex_t::stop()
         }));
     }
 
-    if (get_options()->expire_tiles_zoom_min > 0) {
-        auto const count = output_tiles_to_file(
-            m_expire.get_tiles(), get_options()->expire_tiles_filename.c_str(),
-            get_options()->expire_tiles_zoom_min,
-            get_options()->expire_tiles_zoom);
-        log_info("Wrote {} entries to expired tiles list", count);
+    assert(m_expire_outputs->size() == m_expire_tiles.size());
+    for (std::size_t i = 0; i < m_expire_outputs->size(); ++i) {
+        if (!m_expire_tiles[i].empty()) {
+            auto const &eo = (*m_expire_outputs)[i];
+
+            std::size_t const count = eo.output(m_expire_tiles[i].get_tiles(),
+                                                get_options()->conninfo);
+
+            log_info("Wrote {} entries to expire output [{}].", count, i);
+        }
     }
 }
 
 void output_flex_t::wait()
 {
+    std::exception_ptr eptr;
+    flex_table_t const *table_with_error = nullptr;
+
     for (auto &table : m_table_connections) {
-        table.task_wait();
+        try {
+            table.task_wait();
+        } catch (...) {
+            eptr = std::current_exception();
+            table_with_error = &table.table();
+        }
+    }
+
+    if (eptr) {
+        log_error("Error while doing postprocessing on table '{}':",
+                  table_with_error->name());
+        std::rethrow_exception(eptr);
     }
 }
 
@@ -1295,7 +1104,7 @@ void output_flex_t::node_add(osmium::Node const &node)
     }
 
     m_context_node = &node;
-    get_mutex_and_call_lua_function(m_process_node, node);
+    call_lua_function(m_process_node, node);
     m_context_node = nullptr;
 }
 
@@ -1308,7 +1117,7 @@ void output_flex_t::way_add(osmium::Way *way)
     }
 
     m_way_cache.init(way);
-    get_mutex_and_call_lua_function(m_process_way, m_way_cache.get());
+    call_lua_function(m_process_way, m_way_cache.get());
 }
 
 void output_flex_t::relation_add(osmium::Relation const &relation)
@@ -1319,20 +1128,30 @@ void output_flex_t::relation_add(osmium::Relation const &relation)
 
     m_relation_cache.init(relation);
     select_relation_members();
-    get_mutex_and_call_lua_function(m_process_relation, relation);
+    call_lua_function(m_process_relation, relation);
 }
 
 void output_flex_t::delete_from_table(table_connection_t *table_connection,
                                       osmium::item_type type, osmid_t osm_id)
 {
     assert(table_connection);
-    auto const &table = table_connection->table();
-    auto const id = table.map_id(type, osm_id);
+    auto const id = table_connection->table().map_id(type, osm_id);
 
-    if (m_expire.enabled() && table.has_geom_column() &&
-        table.geom_column().srid() == 3857) {
-        auto const result = table_connection->get_geom_by_id(type, id);
-        expire_from_result(&m_expire, result);
+    if (table_connection->table().has_columns_with_expire()) {
+        auto const result = table_connection->get_geoms_by_id(type, id);
+        auto const num_tuples = result.num_tuples();
+        if (num_tuples > 0) {
+            int col = 0;
+            for (auto const &column : table_connection->table()) {
+                if (column.has_expire()) {
+                    for (int i = 0; i < num_tuples; ++i) {
+                        auto const geom = ewkb_to_geom(result.get(i, col));
+                        column.do_expire(geom, &m_expire_tiles);
+                    }
+                    ++col;
+                }
+            }
+        }
     }
 
     table_connection->delete_rows_with(type, id);
@@ -1392,15 +1211,32 @@ void output_flex_t::start()
     }
 }
 
+static void
+create_expire_tables(std::vector<expire_output_t> const &expire_outputs,
+                     std::string const &conninfo)
+{
+    if (std::all_of(expire_outputs.begin(), expire_outputs.end(),
+                    [](auto const &expire_output) {
+                        return expire_output.table().empty();
+                    })) {
+        return;
+    }
+
+    pg_conn_t const connection{conninfo};
+    for (auto const &expire_output : expire_outputs) {
+        if (!expire_output.table().empty()) {
+            expire_output.create_output_table(connection);
+        }
+    }
+}
+
 output_flex_t::output_flex_t(output_flex_t const *other,
                              std::shared_ptr<middle_query_t> mid,
                              std::shared_ptr<db_copy_thread_t> copy_thread)
 : output_t(other, std::move(mid)), m_tables(other->m_tables),
+  m_expire_outputs(other->m_expire_outputs),
   m_stage2_way_ids(other->m_stage2_way_ids),
   m_copy_thread(std::move(copy_thread)), m_lua_state(other->m_lua_state),
-  m_expire(other->get_options()->expire_tiles_zoom,
-           other->get_options()->expire_tiles_max_bbox,
-           other->get_options()->projection),
   m_process_node(other->m_process_node), m_process_way(other->m_process_way),
   m_process_relation(other->m_process_relation),
   m_select_relation_members(other->m_select_relation_members)
@@ -1410,6 +1246,11 @@ output_flex_t::output_flex_t(output_flex_t const *other,
         tc.connect(get_options()->conninfo);
         tc.prepare();
     }
+
+    for (auto &expire_output : *m_expire_outputs) {
+        m_expire_tiles.emplace_back(expire_output.maxzoom(),
+                                    reprojection::create_projection(3857));
+    }
 }
 
 std::shared_ptr<output_t>
@@ -1423,9 +1264,7 @@ output_flex_t::output_flex_t(std::shared_ptr<middle_query_t> const &mid,
                              std::shared_ptr<thread_pool_t> thread_pool,
                              options_t const &options)
 : output_t(mid, std::move(thread_pool), options),
-  m_copy_thread(std::make_shared<db_copy_thread_t>(options.conninfo)),
-  m_expire(options.expire_tiles_zoom, options.expire_tiles_max_bbox,
-           options.projection)
+  m_copy_thread(std::make_shared<db_copy_thread_t>(options.conninfo))
 {
     init_lua(options.style);
 
@@ -1441,32 +1280,40 @@ output_flex_t::output_flex_t(std::shared_ptr<middle_query_t> const &mid,
             "No tables defined in Lua config. Nothing to do!"};
     }
 
-    log_debug("Tables:");
-    for (auto const &table : *m_tables) {
-        log_debug("- TABLE {}", qualified_name(table.schema(), table.name()));
-        log_debug("  - columns:");
-        for (auto const &column : table) {
-            log_debug(R"(    - "{}" {} ({}) not_null={} create_only={})",
-                      column.name(), column.type_name(), column.sql_type_name(),
-                      column.not_null(), column.create_only());
-        }
-        log_debug("  - data_tablespace={}", table.data_tablespace());
-        log_debug("  - index_tablespace={}", table.index_tablespace());
-        log_debug("  - cluster={}", table.cluster_by_geom());
-        for (auto const &index : table.indexes()) {
-            log_debug("  - INDEX USING {}", index.method());
-            log_debug("    - column={}", index.columns());
-            log_debug("    - expression={}", index.expression());
-            log_debug("    - include={}", index.include_columns());
-            log_debug("    - tablespace={}", index.tablespace());
-            log_debug("    - unique={}", index.is_unique());
-            log_debug("    - where={}", index.where_condition());
+    // For backwards compatibility we add a "default" expire output to all
+    // tables when the relevant command line options are used.
+    if (options.append && options.expire_tiles_zoom) {
+        auto &eo = m_expire_outputs->emplace_back();
+        eo.set_filename(options.expire_tiles_filename);
+        eo.set_minzoom(options.expire_tiles_zoom_min);
+        eo.set_maxzoom(options.expire_tiles_zoom);
+
+        for (auto &table : *m_tables) {
+            if (table.has_geom_column() && table.geom_column().srid() == 3857) {
+                expire_config_t config{};
+                config.expire_output = m_expire_outputs->size() - 1;
+                if (options.expire_tiles_max_bbox > 0.0) {
+                    config.mode = expire_mode::hybrid;
+                    config.full_area_limit = options.expire_tiles_max_bbox;
+                }
+                table.geom_column().add_expire(config);
+            }
         }
     }
 
+    write_expire_output_list_to_debug_log(*m_expire_outputs);
+    write_table_list_to_debug_log(*m_tables);
+
     for (auto &table : *m_tables) {
         m_table_connections.emplace_back(&table, m_copy_thread);
     }
+
+    for (auto &expire_output : *m_expire_outputs) {
+        m_expire_tiles.emplace_back(expire_output.maxzoom(),
+                                    reprojection::create_projection(3857));
+    }
+
+    create_expire_tables(*m_expire_outputs, get_options()->conninfo);
 }
 
 /**
@@ -1493,6 +1340,40 @@ static void init_table_class(lua_State *lua_state)
     luaX_add_table_func(lua_state, "schema", lua_trampoline_table_schema);
     luaX_add_table_func(lua_state, "cluster", lua_trampoline_table_cluster);
     luaX_add_table_func(lua_state, "columns", lua_trampoline_table_columns);
+
+    lua_pop(lua_state, 2);
+}
+
+/**
+ * Define the osm2pgsql.ExpireOutput class/metatable.
+ */
+static void init_expire_output_class(lua_State *lua_state)
+{
+    lua_getglobal(lua_state, "osm2pgsql");
+    if (luaL_newmetatable(lua_state, osm2pgsql_expire_output_name) != 1) {
+        throw std::runtime_error{"Internal error: Lua newmetatable failed."};
+    }
+    lua_pushvalue(lua_state, -1); // Copy of new metatable
+
+    // Add metatable as osm2pgsql.ExpireOutput so we can access it from Lua
+    lua_setfield(lua_state, -3, "ExpireOutput");
+
+    // Now add functions to metatable
+    lua_pushvalue(lua_state, -1);
+    lua_setfield(lua_state, -2, "__index");
+    luaX_add_table_func(lua_state, "__tostring",
+                        lua_trampoline_expire_output_tostring);
+    luaX_add_table_func(lua_state, "minzoom",
+                        lua_trampoline_expire_output_minzoom);
+    luaX_add_table_func(lua_state, "maxzoom",
+                        lua_trampoline_expire_output_maxzoom);
+    luaX_add_table_func(lua_state, "filename",
+                        lua_trampoline_expire_output_filename);
+    luaX_add_table_func(lua_state, "schema",
+                        lua_trampoline_expire_output_schema);
+    luaX_add_table_func(lua_state, "table", lua_trampoline_expire_output_table);
+
+    lua_pop(lua_state, 2);
 }
 
 void output_flex_t::init_lua(std::string const &filename)
@@ -1500,30 +1381,18 @@ void output_flex_t::init_lua(std::string const &filename)
     m_lua_state.reset(luaL_newstate(),
                       [](lua_State *state) { lua_close(state); });
 
-    // Set up global lua libs
-    luaL_openlibs(lua_state());
+    setup_lua_environment(lua_state(), filename, get_options()->append);
 
-    // Set up global "osm2pgsql" object
-    lua_newtable(lua_state());
-
-    luaX_add_table_str(lua_state(), "version", get_osm2pgsql_short_version());
-    luaX_add_table_str(lua_state(), "mode",
-                       get_options()->append ? "append" : "create");
     luaX_add_table_int(lua_state(), "stage", 1);
 
-    std::string dir_path =
-        boost::filesystem::path{filename}.parent_path().string();
-    if (!dir_path.empty()) {
-        dir_path += boost::filesystem::path::preferred_separator;
-    }
-    luaX_add_table_str(lua_state(), "config_dir", dir_path.c_str());
-
     luaX_add_table_func(lua_state(), "define_table",
                         lua_trampoline_app_define_table);
 
-    lua_setglobal(lua_state(), "osm2pgsql");
+    luaX_add_table_func(lua_state(), "define_expire_output",
+                        lua_trampoline_app_define_expire_output);
 
     init_table_class(lua_state());
+    init_expire_output_class(lua_state());
 
     // Clean up stack
     lua_settop(lua_state(), 0);
@@ -1649,7 +1518,7 @@ void output_flex_t::reprocess_marked()
         }
         way_delete(id);
         if (m_process_way) {
-            get_mutex_and_call_lua_function(m_process_way, m_way_cache.get());
+            call_lua_function(m_process_way, m_way_cache.get());
         }
     }
 
@@ -1659,8 +1528,9 @@ void output_flex_t::reprocess_marked()
 
 void output_flex_t::merge_expire_trees(output_t *other)
 {
-    auto *opgsql = dynamic_cast<output_flex_t *>(other);
-    if (opgsql) {
-        m_expire.merge_and_destroy(&opgsql->m_expire);
+    auto *const opgsql = dynamic_cast<output_flex_t *>(other);
+    assert(m_expire_tiles.size() == opgsql->m_expire_tiles.size());
+    for (std::size_t i = 0; i < m_expire_tiles.size(); ++i) {
+        m_expire_tiles[i].merge_and_destroy(&opgsql->m_expire_tiles[i]);
     }
 }
