@@ -10,6 +10,7 @@
  * For a full list of authors see the git log.
  */
 
+#include <cstdint>
 #include <string>
 
 class pg_conn_t;
@@ -20,6 +21,9 @@ bool has_extension(std::string const &value);
 bool has_schema(std::string const &value);
 bool has_tablespace(std::string const &value);
 bool has_index_method(std::string const &value);
+bool has_table(std::string schema, std::string const &name);
+
+void check_schema(std::string const &schema);
 
 /// Get PostgreSQL version in the format (major * 10000 + minor).
 uint32_t get_database_version() noexcept;
