@@ -25,13 +25,6 @@ idlist_t get_ids_from_result(pg_result_t const &result) {
     return ids;
 }
 
-idlist_t get_ids_from_db(pg_conn_t const *db_connection, char const *stmt,
-                         osmid_t id)
-{
-    auto const res = db_connection->exec_prepared(stmt, id);
-    return get_ids_from_result(res);
-}
-
 void create_geom_check_trigger(pg_conn_t *db_connection,
                                std::string const &schema,
                                std::string const &table,
@@ -80,15 +73,3 @@ void analyze_table(pg_conn_t const &db_connection, std::string const &schema,
     auto const qual_name = qualified_name(schema, name);
     db_connection.exec("ANALYZE {}", qual_name);
 }
-
-bool has_table(pg_conn_t const &db_connection, std::string const &schema,
-               std::string const &table)
-{
-    auto const sql = fmt::format("SELECT count(*) FROM pg_tables"
-                                 " WHERE schemaname='{}' AND tablename='{}'",
-                                 schema.empty() ? "public" : schema, table);
-    auto const res = db_connection.exec(sql);
-    char const *const num = res.get_value(0, 0);
-
-    return num[0] == '1' && num[1] == '\0';
-}
