@@ -27,9 +27,6 @@ class pg_result_t;
  */
 idlist_t get_ids_from_result(pg_result_t const &result);
 
-idlist_t get_ids_from_db(pg_conn_t const *db_connection, char const *stmt,
-                         osmid_t id);
-
 void create_geom_check_trigger(pg_conn_t *db_connection,
                                std::string const &schema,
                                std::string const &table,
@@ -42,11 +39,4 @@ void drop_geom_check_trigger(pg_conn_t *db_connection,
 void analyze_table(pg_conn_t const &db_connection, std::string const &schema,
                    std::string const &name);
 
-/**
- * Check whether the table with the specified name exists in the specified
- * schema in the database. Leave schema empty to check in the 'public' schema.
- */
-bool has_table(pg_conn_t const &db_connection, std::string const &schema,
-               std::string const &table);
-
 #endif // OSM2PGSQL_PGSQL_HELPER_HPP
