@@ -13,8 +13,6 @@
 /**
  * \file
  *
- * This file is part of osm2pgsql (https://github.com/openstreetmap/osm2pgsql).
- *
  * Helper classes and functions for PostgreSQL access.
  */
 
@@ -25,9 +23,11 @@
 #include <array>
 #include <atomic>
 #include <cassert>
+#include <cstdint>
 #include <map>
 #include <memory>
 #include <string>
+#include <string_view>
 #include <vector>
 
 /**
@@ -66,6 +66,9 @@ public:
         return PQgetisnull(m_result.get(), row, col) != 0;
     }
 
+    /// Return the number of INSERTed, UPDATEd, or DELETEed rows.
+    std::size_t affected_rows() const noexcept;
+
     /**
      * The length of the field at (row, col) in bytes.
      *
@@ -123,6 +126,20 @@ private:
     std::unique_ptr<PGresult, pg_result_deleter_t> m_result;
 };
 
+/**
+ * Wrapper class for query parameters that should be sent to the database
+ * as binary parameter.
+ */
+class binary_param : public std::string_view
+{
+public:
+    using std::string_view::string_view;
+
+    explicit binary_param(std::string const &str)
+    : std::string_view(str.data(), str.size())
+    {}
+};
+
 /**
  * PostgreSQL connection.
  *
@@ -155,7 +172,7 @@ public:
      *         status code PGRES_COMMAND_OK or PGRES_TUPLES_OK).
      */
     template <typename... TArgs>
-    pg_result_t exec(char const *sql, TArgs... params) const
+    pg_result_t exec(fmt::format_string<TArgs...> sql, TArgs... params) const
     {
         return exec(fmt::format(sql, std::forward<TArgs>(params)...));
     }
@@ -222,9 +239,9 @@ private:
     template <typename T>
     static constexpr std::size_t buffers_needed() noexcept
     {
-        if constexpr (std::is_same_v<T, char const *>) {
-            return 0;
-        } else if constexpr (std::is_same_v<T, std::string>) {
+        if constexpr (std::is_same_v<T, char const *> ||
+                      std::is_same_v<T, std::string> ||
+                      std::is_same_v<T, binary_param>) {
             return 0;
         }
         return 1;
@@ -237,12 +254,18 @@ private:
      * strings.
      */
     template <typename T>
-    static char const *to_str(std::vector<std::string> *data, T const &param)
+    static char const *to_str(std::vector<std::string> *data, int *length,
+                              int *bin, T const &param)
     {
         if constexpr (std::is_same_v<T, char const *>) {
             return param;
         } else if constexpr (std::is_same_v<T, std::string>) {
+            *length = param.size();
             return param.c_str();
+        } else if constexpr (std::is_same_v<T, binary_param>) {
+            *length = param.size();
+            *bin = 1;
+            return param.data();
         }
         return data->emplace_back(fmt::to_string(param)).c_str();
     }
@@ -272,16 +295,22 @@ private:
         std::vector<std::string> exec_params;
         exec_params.reserve(total_buffers_needed);
 
+        std::array<int, sizeof...(params)> lengths = {0};
+        std::array<int, sizeof...(params)> bins = {0};
+
         // This array holds the pointers to all parameter strings, either
         // to the original string parameters or to the recently converted
         // in the exec_params vector.
+        std::size_t n = 0;
+        std::size_t m = 0;
         std::array<char const *, sizeof...(params)> param_ptrs = {
-            to_str<std::decay_t<TArgs>>(&exec_params,
+            to_str<std::decay_t<TArgs>>(&exec_params, &lengths.at(n++),
+                                        &bins.at(m++),
                                         std::forward<TArgs>(params))...};
 
         return exec_prepared_internal(stmt, sizeof...(params),
-                                      param_ptrs.data(), nullptr, nullptr,
-                                      result_as_binary ? 1 : 0);
+                                      param_ptrs.data(), lengths.data(),
+                                      bins.data(), result_as_binary ? 1 : 0);
     }
 
     struct pg_conn_deleter_t
