@@ -13,12 +13,11 @@
 /**
  * \file
  *
- * This file is part of osm2pgsql (https://github.com/openstreetmap/osm2pgsql).
- *
  * It contains the progress_display_t class.
  */
 
 #include <cstddef>
+#include <cstdint>
 #include <ctime>
 
 #include <osmium/handler.hpp>
