@@ -6,7 +6,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2022 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -22,8 +22,8 @@ enum class ColumnType
 
 struct Column
 {
-    Column(std::string const &n, std::string const &tn, ColumnType t)
-    : name(n), type_name(tn), type(t)
+    Column(std::string n, std::string tn, ColumnType t)
+    : name(std::move(n)), type_name(std::move(tn)), type(t)
     {}
 
     std::string name;
