@@ -10,8 +10,10 @@
  * For a full list of authors see the git log.
  */
 
+#include "geom-box.hpp"
 #include "geom.hpp"
 
+#include <algorithm>
 #include <cstdint>
 #include <limits>
 
@@ -143,6 +145,56 @@ public:
         return half_earth_circumference - m_y * extent();
     }
 
+    /// Same as box(margin).min_x().
+    double xmin(double margin) const noexcept
+    {
+        return std::clamp(xmin() - margin * extent(), -half_earth_circumference,
+                          half_earth_circumference);
+    }
+
+    /// Same as box(margin).max_x().
+    double xmax(double margin) const noexcept
+    {
+        return std::clamp(xmax() + margin * extent(), -half_earth_circumference,
+                          half_earth_circumference);
+    }
+
+    /// Same as box(margin).min_y().
+    double ymin(double margin) const noexcept
+    {
+        return std::clamp(ymin() - margin * extent(), -half_earth_circumference,
+                          half_earth_circumference);
+    }
+
+    /// Same as box(margin).max_y().
+    double ymax(double margin) const noexcept
+    {
+        return std::clamp(ymax() + margin * extent(), -half_earth_circumference,
+                          half_earth_circumference);
+    }
+
+    /**
+     * Get bounding box from tile.
+     */
+    geom::box_t box() const noexcept
+    {
+        return {xmin(), ymin(), xmax(), ymax()};
+    }
+
+    /**
+     * Get bounding box from tile with margin on all sides. The margin is
+     * specified as a fraction of the tile extent. So margin==0.5 (50%) makes
+     * the bounding box twice as wide and twice as heigh.
+     *
+     * The bounding box is clamped to the extent of the earth, so there will
+     * be no coordinates smaller than -half_earth_circumference or larger than
+     * half_earth_circumference.
+     */
+    geom::box_t box(double margin) const noexcept
+    {
+        return {xmin(margin), ymin(margin), xmax(margin), ymax(margin)};
+    }
+
     /**
      * Convert a point from web mercator (EPSG:3857) coordinates to
      * coordinates in the tile assuming a tile extent of `pixel_extent`.
@@ -215,4 +267,55 @@ private:
     uint32_t m_zoom = invalid_zoom;
 }; // class tile_t
 
+/**
+ * Iterate over tiles and call output function for each tile on all requested
+ * zoom levels.
+ *
+ * \tparam OUTPUT Class with operator() taking a tile_t argument
+ *
+ * \param tiles_at_maxzoom The list of tiles at maximum zoom level
+ * \param minzoom Minimum zoom level
+ * \param maxzoom Maximum zoom level
+ * \param output Output function
+ */
+template <class OUTPUT>
+std::size_t for_each_tile(quadkey_list_t const &tiles_at_maxzoom,
+                          uint32_t minzoom, uint32_t maxzoom, OUTPUT &&output)
+{
+    assert(minzoom <= maxzoom);
+
+    if (minzoom == maxzoom) {
+        for (auto const quadkey : tiles_at_maxzoom) {
+            std::forward<OUTPUT>(output)(
+                tile_t::from_quadkey(quadkey, maxzoom));
+        }
+        return tiles_at_maxzoom.size();
+    }
+
+    /**
+     * Loop over all requested zoom levels (from maximum down to the minimum
+     * zoom level).
+     */
+    quadkey_t last_quadkey{};
+    std::size_t count = 0;
+    for (auto const quadkey : tiles_at_maxzoom) {
+        for (uint32_t dz = 0; dz <= maxzoom - minzoom; ++dz) {
+            auto const qt_current = quadkey.down(dz);
+            /**
+             * If dz > 0, there are probably multiple elements whose quadkey
+             * is equal because they are all sub-tiles of the same tile at the
+             * current zoom level. We skip all of them after we have written
+             * the first sibling.
+             */
+            if (qt_current != last_quadkey.down(dz)) {
+                std::forward<OUTPUT>(output)(
+                    tile_t::from_quadkey(qt_current, maxzoom - dz));
+                ++count;
+            }
+        }
+        last_quadkey = quadkey;
+    }
+    return count;
+}
+
 #endif // OSM2PGSQL_TILE_HPP
