@@ -10,6 +10,7 @@
 #include "wkb.hpp"
 
 #include "format.hpp"
+#include "overloaded.hpp"
 
 #include <array>
 #include <cassert>
@@ -231,13 +232,13 @@ public:
 
         if (m_ensure_multi) {
             // Two 13 bytes headers plus n sets of coordinates
-            data.reserve(2 * 13 + geom.size() * (2 * 8));
+            data.reserve(2UL * 13UL + geom.size() * (2UL * 8UL));
             write_header(&data, wkb_multi_line, m_srid);
             write_length(&data, 1);
             write_linestring(&data, geom);
         } else {
             // 13 byte header plus n sets of coordinates
-            data.reserve(13 + geom.size() * (2 * 8));
+            data.reserve(13UL + geom.size() * (2UL * 8UL));
             write_linestring(&data, geom, m_srid);
         }
 
@@ -306,7 +307,7 @@ private:
 class ewkb_parser_t
 {
 public:
-    ewkb_parser_t(std::string_view input)
+    explicit ewkb_parser_t(std::string_view input)
     : m_data(input),
       m_max_length(static_cast<uint32_t>(input.size()) / (sizeof(double) * 2))
     {}
