@@ -26,19 +26,20 @@ static constexpr uint32_t const zoom = 12;
 
 TEST_CASE("expire null geometry does nothing", "[NoDB]")
 {
-    expire_tiles et{zoom, 20000, defproj};
+    expire_config_t const expire_config;
+    expire_tiles et{zoom, defproj};
 
     SECTION("geom")
     {
         geom::geometry_t const geom{};
-        et.from_geometry(geom);
+        et.from_geometry(geom, expire_config);
     }
 
     SECTION("geom with check")
     {
         geom::geometry_t geom{};
         geom.set_srid(3857);
-        et.from_geometry_if_3857(geom);
+        et.from_geometry_if_3857(geom, expire_config);
     }
 
     REQUIRE(et.get_tiles().empty());
@@ -46,23 +47,24 @@ TEST_CASE("expire null geometry does nothing", "[NoDB]")
 
 TEST_CASE("expire point at tile boundary", "[NoDB]")
 {
-    expire_tiles et{zoom, 20000, defproj};
+    expire_config_t const expire_config;
+    expire_tiles et{zoom, defproj};
 
     geom::point_t const pt{0.0, 0.0};
 
-    SECTION("point") { et.from_geometry(pt); }
+    SECTION("point") { et.from_geometry(pt, expire_config); }
 
     SECTION("geom")
     {
         geom::geometry_t const geom{pt};
-        et.from_geometry(geom);
+        et.from_geometry(geom, expire_config);
     }
 
     SECTION("geom with check")
     {
         geom::geometry_t geom{pt};
         geom.set_srid(3857);
-        et.from_geometry_if_3857(geom);
+        et.from_geometry_if_3857(geom, expire_config);
     }
 
     auto const tiles = et.get_tiles();
@@ -75,23 +77,24 @@ TEST_CASE("expire point at tile boundary", "[NoDB]")
 
 TEST_CASE("expire point away from tile boundary", "[NoDB]")
 {
-    expire_tiles et{zoom, 20000, defproj};
+    expire_config_t const expire_config;
+    expire_tiles et{zoom, defproj};
 
     geom::point_t const pt{5000.0, 5000.0};
 
-    SECTION("point") { et.from_geometry(pt); }
+    SECTION("point") { et.from_geometry(pt, expire_config); }
 
     SECTION("geom")
     {
         geom::geometry_t const geom{pt};
-        et.from_geometry(geom);
+        et.from_geometry(geom, expire_config);
     }
 
     SECTION("geom with check")
     {
         geom::geometry_t geom{pt};
         geom.set_srid(3857);
-        et.from_geometry_if_3857(geom);
+        et.from_geometry_if_3857(geom, expire_config);
     }
 
     auto const tiles = et.get_tiles();
@@ -101,19 +104,20 @@ TEST_CASE("expire point away from tile boundary", "[NoDB]")
 
 TEST_CASE("expire linestring away from tile boundary", "[NoDB]")
 {
-    expire_tiles et{zoom, 20000, defproj};
+    expire_config_t const expire_config;
+    expire_tiles et{zoom, defproj};
 
     SECTION("line")
     {
         geom::linestring_t const line{{5000.0, 4000.0}, {5100.0, 4200.0}};
-        et.from_geometry(line);
+        et.from_geometry(line, expire_config);
     }
 
     SECTION("geom")
     {
         geom::linestring_t line{{5000.0, 4000.0}, {5100.0, 4200.0}};
         geom::geometry_t const geom{std::move(line)};
-        et.from_geometry(geom);
+        et.from_geometry(geom, expire_config);
     }
 
     SECTION("geom with check")
@@ -121,7 +125,7 @@ TEST_CASE("expire linestring away from tile boundary", "[NoDB]")
         geom::linestring_t line{{5000.0, 4000.0}, {5100.0, 4200.0}};
         geom::geometry_t geom{std::move(line)};
         geom.set_srid(3857);
-        et.from_geometry_if_3857(geom);
+        et.from_geometry_if_3857(geom, expire_config);
     }
 
     auto const tiles = et.get_tiles();
@@ -131,19 +135,20 @@ TEST_CASE("expire linestring away from tile boundary", "[NoDB]")
 
 TEST_CASE("expire linestring crossing tile boundary", "[NoDB]")
 {
-    expire_tiles et{zoom, 20000, defproj};
+    expire_config_t const expire_config;
+    expire_tiles et{zoom, defproj};
 
     SECTION("line")
     {
         geom::linestring_t const line{{5000.0, 5000.0}, {5000.0, 15000.0}};
-        et.from_geometry(line);
+        et.from_geometry(line, expire_config);
     }
 
     SECTION("geom")
     {
         geom::linestring_t line{{5000.0, 5000.0}, {5000.0, 15000.0}};
         geom::geometry_t const geom{std::move(line)};
-        et.from_geometry(geom);
+        et.from_geometry(geom, expire_config);
     }
 
     SECTION("geom with check")
@@ -151,7 +156,7 @@ TEST_CASE("expire linestring crossing tile boundary", "[NoDB]")
         geom::linestring_t line{{5000.0, 5000.0}, {5000.0, 15000.0}};
         geom::geometry_t geom{std::move(line)};
         geom.set_srid(3857);
-        et.from_geometry_if_3857(geom);
+        et.from_geometry_if_3857(geom, expire_config);
     }
 
     auto const tiles = et.get_tiles();
@@ -162,7 +167,8 @@ TEST_CASE("expire linestring crossing tile boundary", "[NoDB]")
 
 TEST_CASE("expire small polygon", "[NoDB]")
 {
-    expire_tiles et{zoom, 20000, defproj};
+    expire_config_t const expire_config;
+    expire_tiles et{zoom, defproj};
 
     SECTION("polygon")
     {
@@ -171,7 +177,7 @@ TEST_CASE("expire small polygon", "[NoDB]")
                                     {5100.0, 5100.0},
                                     {5000.0, 5100.0},
                                     {5000.0, 5000.0}}};
-        et.from_geometry(poly);
+        et.from_geometry(poly, expire_config);
     }
 
     SECTION("geom")
@@ -182,7 +188,7 @@ TEST_CASE("expire small polygon", "[NoDB]")
                               {5000.0, 5100.0},
                               {5000.0, 5000.0}}};
         geom::geometry_t const geom{std::move(poly)};
-        et.from_geometry(geom);
+        et.from_geometry(geom, expire_config);
     }
 
     SECTION("geom with check")
@@ -194,7 +200,7 @@ TEST_CASE("expire small polygon", "[NoDB]")
                               {5000.0, 5000.0}}};
         geom::geometry_t geom{std::move(poly)};
         geom.set_srid(3857);
-        et.from_geometry_if_3857(geom);
+        et.from_geometry_if_3857(geom, expire_config);
     }
 
     auto const tiles = et.get_tiles();
@@ -204,7 +210,10 @@ TEST_CASE("expire small polygon", "[NoDB]")
 
 TEST_CASE("expire large polygon as bbox", "[NoDB]")
 {
-    expire_tiles et{zoom, 40000, defproj};
+    expire_config_t expire_config;
+    expire_config.mode = expire_mode::hybrid;
+    expire_config.full_area_limit = 40000;
+    expire_tiles et{zoom, defproj};
 
     SECTION("polygon")
     {
@@ -213,7 +222,7 @@ TEST_CASE("expire large polygon as bbox", "[NoDB]")
                                     {25000.0, 25000.0},
                                     {5000.0, 25000.0},
                                     {5000.0, 5000.0}}};
-        et.from_geometry(poly);
+        et.from_geometry(poly, expire_config);
     }
 
     SECTION("geom")
@@ -224,7 +233,7 @@ TEST_CASE("expire large polygon as bbox", "[NoDB]")
                               {5000.0, 25000.0},
                               {5000.0, 5000.0}}};
         geom::geometry_t const geom{std::move(poly)};
-        et.from_geometry(geom);
+        et.from_geometry(geom, expire_config);
     }
 
     SECTION("geom with check")
@@ -236,7 +245,7 @@ TEST_CASE("expire large polygon as bbox", "[NoDB]")
                               {5000.0, 5000.0}}};
         geom::geometry_t geom{std::move(poly)};
         geom.set_srid(3857);
-        et.from_geometry_if_3857(geom);
+        et.from_geometry_if_3857(geom, expire_config);
     }
 
     auto const tiles = et.get_tiles();
@@ -256,7 +265,10 @@ TEST_CASE("expire large polygon as bbox", "[NoDB]")
 
 TEST_CASE("expire large polygon as boundary", "[NoDB]")
 {
-    expire_tiles et{zoom, 10000, defproj};
+    expire_config_t expire_config;
+    expire_config.mode = expire_mode::hybrid;
+    expire_config.full_area_limit = 10000;
+    expire_tiles et{zoom, defproj};
 
     SECTION("polygon")
     {
@@ -265,7 +277,7 @@ TEST_CASE("expire large polygon as boundary", "[NoDB]")
                                     {25000.0, 25000.0},
                                     {5000.0, 25000.0},
                                     {5000.0, 5000.0}}};
-        et.from_geometry(poly);
+        et.from_geometry(poly, expire_config);
     }
 
     SECTION("polygon boundary")
@@ -275,7 +287,7 @@ TEST_CASE("expire large polygon as boundary", "[NoDB]")
                                     {25000.0, 25000.0},
                                     {5000.0, 25000.0},
                                     {5000.0, 5000.0}}};
-        et.from_polygon_boundary(poly);
+        et.from_polygon_boundary(poly, expire_config);
     }
 
     SECTION("geom")
@@ -286,7 +298,7 @@ TEST_CASE("expire large polygon as boundary", "[NoDB]")
                               {5000.0, 25000.0},
                               {5000.0, 5000.0}}};
         geom::geometry_t const geom{std::move(poly)};
-        et.from_geometry(geom);
+        et.from_geometry(geom, expire_config);
     }
 
     SECTION("geom with check")
@@ -298,7 +310,7 @@ TEST_CASE("expire large polygon as boundary", "[NoDB]")
                               {5000.0, 5000.0}}};
         geom::geometry_t geom{std::move(poly)};
         geom.set_srid(3857);
-        et.from_geometry_if_3857(geom);
+        et.from_geometry_if_3857(geom, expire_config);
     }
 
     auto const tiles = et.get_tiles();
@@ -317,36 +329,37 @@ TEST_CASE("expire large polygon as boundary", "[NoDB]")
 
 TEST_CASE("expire multipoint geometry", "[NoDB]")
 {
-    expire_tiles et{zoom, 20000, defproj};
+    expire_config_t const expire_config;
+    expire_tiles et{zoom, defproj};
 
-    geom::point_t p1{0.0, 0.0};
-    geom::point_t p2{15000.0, 15000.0};
+    geom::point_t const p1{0.0, 0.0};
+    geom::point_t const p2{15000.0, 15000.0};
 
     SECTION("multipoint")
     {
         geom::multipoint_t mpt;
-        mpt.add_geometry(std::move(p1));
-        mpt.add_geometry(std::move(p2));
-        et.from_geometry(mpt);
+        mpt.add_geometry(p1);
+        mpt.add_geometry(p2);
+        et.from_geometry(mpt, expire_config);
     }
 
     SECTION("geom")
     {
         geom::multipoint_t mpt;
-        mpt.add_geometry(std::move(p1));
-        mpt.add_geometry(std::move(p2));
+        mpt.add_geometry(p1);
+        mpt.add_geometry(p2);
         geom::geometry_t const geom{std::move(mpt)};
-        et.from_geometry(geom);
+        et.from_geometry(geom, expire_config);
     }
 
     SECTION("geom with check")
     {
         geom::multipoint_t mpt;
-        mpt.add_geometry(std::move(p1));
-        mpt.add_geometry(std::move(p2));
+        mpt.add_geometry(p1);
+        mpt.add_geometry(p2);
         geom::geometry_t geom{std::move(mpt)};
         geom.set_srid(3857);
-        et.from_geometry_if_3857(geom);
+        et.from_geometry_if_3857(geom, expire_config);
     }
 
     auto const tiles = et.get_tiles();
@@ -360,7 +373,8 @@ TEST_CASE("expire multipoint geometry", "[NoDB]")
 
 TEST_CASE("expire multilinestring geometry", "[NoDB]")
 {
-    expire_tiles et{zoom, 20000, defproj};
+    expire_config_t const expire_config;
+    expire_tiles et{zoom, defproj};
 
     geom::linestring_t l1{{2000.0, 2000.0}, {3000.0, 3000.0}};
     geom::linestring_t l2{{15000.0, 15000.0}, {25000.0, 15000.0}};
@@ -368,19 +382,19 @@ TEST_CASE("expire multilinestring geometry", "[NoDB]")
     ml.add_geometry(std::move(l1));
     ml.add_geometry(std::move(l2));
 
-    SECTION("multilinestring") { et.from_geometry(ml); }
+    SECTION("multilinestring") { et.from_geometry(ml, expire_config); }
 
     SECTION("geom")
     {
         geom::geometry_t const geom{std::move(ml)};
-        et.from_geometry(geom);
+        et.from_geometry(geom, expire_config);
     }
 
     SECTION("geom with check")
     {
         geom::geometry_t geom{std::move(ml)};
         geom.set_srid(3857);
-        et.from_geometry_if_3857(geom);
+        et.from_geometry_if_3857(geom, expire_config);
     }
 
     auto const tiles = et.get_tiles();
@@ -392,7 +406,10 @@ TEST_CASE("expire multilinestring geometry", "[NoDB]")
 
 TEST_CASE("expire multipolygon geometry", "[NoDB]")
 {
-    expire_tiles et{zoom, 10000, defproj};
+    expire_config_t expire_config;
+    expire_config.mode = expire_mode::hybrid;
+    expire_config.full_area_limit = 10000;
+    expire_tiles et{zoom, defproj};
 
     geom::polygon_t p1{{{2000.0, 2000.0},
                         {2000.0, 3000.0},
@@ -415,19 +432,19 @@ TEST_CASE("expire multipolygon geometry", "[NoDB]")
     mp.add_geometry(std::move(p1));
     mp.add_geometry(std::move(p2));
 
-    SECTION("multilinestring") { et.from_geometry(mp); }
+    SECTION("multilinestring") { et.from_geometry(mp, expire_config); }
 
     SECTION("geom")
     {
         geom::geometry_t const geom{std::move(mp)};
-        et.from_geometry(geom);
+        et.from_geometry(geom, expire_config);
     }
 
     SECTION("geom with check")
     {
         geom::geometry_t geom{std::move(mp)};
         geom.set_srid(3857);
-        et.from_geometry_if_3857(geom);
+        et.from_geometry_if_3857(geom, expire_config);
     }
 
     auto const tiles = et.get_tiles();
@@ -451,7 +468,8 @@ TEST_CASE("expire multipolygon geometry", "[NoDB]")
 
 TEST_CASE("expire geometry collection", "[NoDB]")
 {
-    expire_tiles et{zoom, 20000, defproj};
+    expire_config_t const expire_config;
+    expire_tiles et{zoom, defproj};
 
     geom::collection_t collection;
     collection.add_geometry(geom::geometry_t{geom::point_t{0.0, 0.0}});
@@ -461,14 +479,14 @@ TEST_CASE("expire geometry collection", "[NoDB]")
     SECTION("geom")
     {
         geom::geometry_t const geom{std::move(collection)};
-        et.from_geometry(geom);
+        et.from_geometry(geom, expire_config);
     }
 
     SECTION("geom with check")
     {
         geom::geometry_t geom{std::move(collection)};
         geom.set_srid(3857);
-        et.from_geometry_if_3857(geom);
+        et.from_geometry_if_3857(geom, expire_config);
     }
 
     auto const tiles = et.get_tiles();
@@ -483,11 +501,12 @@ TEST_CASE("expire geometry collection", "[NoDB]")
 
 TEST_CASE("expire doesn't do anything if not in 3857", "[NoDB]")
 {
-    expire_tiles et{zoom, 20000, defproj};
+    expire_config_t const expire_config;
+    expire_tiles et{zoom, defproj};
 
     geom::geometry_t geom{geom::point_t{0.0, 0.0}};
     geom.set_srid(1234);
-    et.from_geometry_if_3857(geom);
+    et.from_geometry_if_3857(geom, expire_config);
 
     auto const tiles = et.get_tiles();
     REQUIRE(tiles.empty());
