@@ -12,6 +12,8 @@
 #include <vector>
 
 #include "common-import.hpp"
+
+#include "command-line-parser.hpp"
 #include "reprojection.hpp"
 
 static testing::db::import_t db;
@@ -73,8 +75,8 @@ TEST_CASE("Projection setup")
 
     option_params.push_back("foo");
 
-    options_t const options{(int)option_params.size(),
-                            (char **)option_params.data()};
+    auto const options = parse_command_line((int)option_params.size(),
+                                            (char **)option_params.data());
 
     if (!proj_name.empty()) {
         CHECK(options.projection->target_desc() == proj_name);
