@@ -3,7 +3,7 @@
  *
  * This file is part of osm2pgsql (https://osm2pgsql.org/).
  *
- * Copyright (C) 2006-2021 by the osm2pgsql developer community.
+ * Copyright (C) 2006-2023 by the osm2pgsql developer community.
  * For a full list of authors see the git log.
  */
 
@@ -35,7 +35,7 @@ struct options_slim_latlon
 TEMPLATE_TEST_CASE("liechtenstein regression", "", options_slim_default,
                    options_slim_latlon)
 {
-    options_t options = TestType::options();
+    options_t const options = TestType::options();
 
     REQUIRE_NOTHROW(db.run_file(options, "liechtenstein-2013-08-03.osm.pbf"));
 
