@@ -26,8 +26,8 @@ TEST_CASE("simple import with tables spaces")
 {
     {
         auto conn = db.db().connect();
-        REQUIRE(1 ==
-                conn.get_count("pg_tablespace", "spcname = 'tablespacetest'"));
+        REQUIRE(1 == conn.get_count("pg_catalog.pg_tablespace",
+                                    "spcname = 'tablespacetest'"));
     }
 
     options_t options = testing::opt_t().slim();
