@@ -24,9 +24,9 @@ TEST_CASE("Tablespace clause with tablespace")
     REQUIRE(tablespace_clause("foo") == R"( TABLESPACE "foo")");
 }
 
-TEST_CASE("Table name without schema")
+TEST_CASE("Table name with public schema")
 {
-    REQUIRE(qualified_name("", "foo") == R"("foo")");
+    REQUIRE(qualified_name("public", "foo") == R"("public"."foo")");
 }
 
 TEST_CASE("Table name with schema")
@@ -36,7 +36,7 @@ TEST_CASE("Table name with schema")
 
 TEST_CASE("query with SELECT should work")
 {
-    auto conn = db.db().connect();
+    auto const conn = db.db().connect();
     auto const result = conn.exec("SELECT 42");
     REQUIRE(result.status() == PGRES_TUPLES_OK);
     REQUIRE(result.num_fields() == 1);
@@ -46,31 +46,19 @@ TEST_CASE("query with SELECT should work")
 
 TEST_CASE("query with invalid SQL should fail")
 {
-    auto conn = db.db().connect();
+    auto const conn = db.db().connect();
     REQUIRE_THROWS(conn.exec("NOT-VALID-SQL"));
 }
 
 TEST_CASE("exec with invalid SQL should fail")
 {
-    auto conn = db.db().connect();
+    auto const conn = db.db().connect();
     REQUIRE_THROWS(conn.exec("XYZ"));
 }
 
-TEST_CASE("exec_prepared without parameters should work")
-{
-    auto conn = db.db().connect();
-    conn.exec("PREPARE test AS SELECT 42");
-
-    auto const result = conn.exec_prepared("test");
-    REQUIRE(result.status() == PGRES_TUPLES_OK);
-    REQUIRE(result.num_fields() == 1);
-    REQUIRE(result.num_tuples() == 1);
-    REQUIRE(result.get(0, 0) == "42");
-}
-
 TEST_CASE("exec_prepared with single string parameters should work")
 {
-    auto conn = db.db().connect();
+    auto const conn = db.db().connect();
     conn.exec("PREPARE test(int) AS SELECT $1");
 
     auto const result = conn.exec_prepared("test", "17");
@@ -82,7 +70,7 @@ TEST_CASE("exec_prepared with single string parameters should work")
 
 TEST_CASE("exec_prepared with string parameters should work")
 {
-    auto conn = db.db().connect();
+    auto const conn = db.db().connect();
     conn.exec("PREPARE test(int, int, int, int, int)"
               " AS SELECT $1 + $2 + $3 + $4 + $5");
 
@@ -98,7 +86,7 @@ TEST_CASE("exec_prepared with string parameters should work")
 
 TEST_CASE("exec_prepared with non-string parameters should work")
 {
-    auto conn = db.db().connect();
+    auto const conn = db.db().connect();
     conn.exec("PREPARE test(int, int, int) AS SELECT $1 + $2 + $3");
 
     auto const result = conn.exec_prepared("test", 1, 2.0, 3ULL);
@@ -107,3 +95,43 @@ TEST_CASE("exec_prepared with non-string parameters should work")
     REQUIRE(result.num_tuples() == 1);
     REQUIRE(result.get(0, 0) == "6");
 }
+
+TEST_CASE("exec_prepared with binary parameter should work")
+{
+    auto const conn = db.db().connect();
+    conn.exec("PREPARE test(bytea) AS SELECT length($1)");
+
+    binary_param const p{"foo \x01 bar"};
+    auto const result = conn.exec_prepared("test", p);
+    REQUIRE(result.status() == PGRES_TUPLES_OK);
+    REQUIRE(result.num_fields() == 1);
+    REQUIRE(result.num_tuples() == 1);
+    REQUIRE(result.get(0, 0) == "9");
+}
+
+TEST_CASE("exec_prepared with mixed parameter types should work")
+{
+    auto const conn = db.db().connect();
+    conn.exec("PREPARE test(text, bytea, int) AS"
+              " SELECT length($1) + length($2) + $3");
+
+    std::string const p1{"foo bar"};
+    binary_param const p2{"foo \x01 bar"};
+    int const p3 = 17;
+    auto const result = conn.exec_prepared("test", p1, p2, p3);
+    REQUIRE(result.status() == PGRES_TUPLES_OK);
+    REQUIRE(result.num_fields() == 1);
+    REQUIRE(result.num_tuples() == 1);
+    REQUIRE(result.get(0, 0) == "33"); // 7 + 9 + 17
+}
+
+TEST_CASE("create table and insert something")
+{
+    auto const conn = db.db().connect();
+    conn.exec("CREATE TABLE foo (x int)");
+    auto const result = conn.exec("INSERT INTO foo (x) VALUES (1), (2)");
+    REQUIRE(result.status() == PGRES_COMMAND_OK);
+    REQUIRE(result.num_fields() == 0);
+    REQUIRE(result.num_tuples() == 0);
+    REQUIRE(result.affected_rows() == 2);
+}
